package com.hazelcast.swarm;

import java.util.List;
import java.util.Map;

import com.hazelcast.logging.ILogger;
import com.hazelcast.spi.discovery.AbstractDiscoveryStrategy;
import com.hazelcast.spi.discovery.DiscoveryNode;

final class HazelcastSwarmDiscoveryStrategy extends AbstractDiscoveryStrategy {
    private final EndpointResolver endpointResolver;

    HazelcastSwarmDiscoveryStrategy(final ILogger logger, final Map<String, Comparable> properties) {
        super(logger, properties);

        EndpointResolver endpointResolver;
        endpointResolver = new ServiceEndpointResolver(logger);
        logger.info("Swarm Discovery activated resolver: " + endpointResolver.getClass().getSimpleName());
        this.endpointResolver = endpointResolver;
    }

    @Override
    public void start() {
        endpointResolver.start();
    }

    @Override
    public Iterable<DiscoveryNode> discoverNodes() {
        return endpointResolver.resolve();
    }

    @Override
    public void destroy() {
        endpointResolver.destroy();
    }

    abstract static class EndpointResolver {
        protected final ILogger logger;

        EndpointResolver(final ILogger logger) {
            this.logger = logger;
        }

        abstract List<DiscoveryNode> resolve();

        void start() {
        }

        void destroy() {
        }
    }
}
