package com.hazelcast.swarm;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import com.hazelcast.logging.ILogger;
import com.hazelcast.nio.Address;
import com.hazelcast.spi.discovery.DiscoveryNode;
import com.hazelcast.spi.discovery.SimpleDiscoveryNode;

class ServiceEndpointResolver extends HazelcastSwarmDiscoveryStrategy.EndpointResolver {
    protected final ILogger logger;
    
    ServiceEndpointResolver(final ILogger logger) {
        super(logger);
        this.logger = logger;
    }

    public InetAddress[] queryAllByName(final String name) throws UnknownHostException {
        final InetAddress[] addresses = InetAddress.getAllByName(name);

        return addresses;
    }

    public void waitForDnsAvailable() { // 「tasks.payara-service」で IP が引けるようになるまで待機
        for (;;) {
            try {
                final InetAddress[] addresses = queryAllByName("tasks.payara-service");

                if (addresses != null) {
                    break;
                }
            } catch (final Exception ignore) {
            }

            try { TimeUnit.SECONDS.sleep(1); } catch (final InterruptedException ignore) {}
        }
    }

    @Override
    List<DiscoveryNode> resolve() {
        waitForDnsAvailable();

        try {
            System.out.println("--------------------------------------------");
            final List<DiscoveryNode> result = new ArrayList<DiscoveryNode>();
            final InetAddress[] addresses = queryAllByName("tasks.payara-service");

            for (final InetAddress address : addresses) {
                System.out.println("Discoved node: " + address.getHostAddress());
                logger.info("Discoved node: " + address.getHostAddress());
                
                result.add(makeDiscoveryNode(address.getHostAddress(), 5701)); // DAS
                result.add(makeDiscoveryNode(address.getHostAddress(), 5702)); // インスタンス
            }

            return result;
        } catch (final Exception e) {
            throw new RuntimeException(e.getLocalizedMessage(), e);
        }
    }

    DiscoveryNode makeDiscoveryNode(final String host, final int port) throws UnknownHostException {
        final Address address = new Address(host, port);
        final SimpleDiscoveryNode discoveryNode = new SimpleDiscoveryNode(address, address);

        return discoveryNode;
    }
}
